import fsmMgr

import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import mouseInfo
import stateAim
import playerMgr
import tutorialProgress
import levelIntros
import stateDialog
import stateMainMenu
import stateLevelScore
import soundMgr
import statePutt
import options
import recordShot
import textDraw
import miniSim

LOC_INCREMENT = 0.1
DRAW_ACCELERATION = False

BASE_PING_PERIOD = 0.1

class StateFlight(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.angle=0 #east, I guess

        self.curPlan={"radius":40,
                      "center":(80,80)}

        self.flightTime=0
        self.locs=[]
        self.exitMouseDown=False
        self.lastPingTime=0.0

    def init(self):
        curPlayer=playerMgr.getCurrentPlayer()
        shotNum=curPlayer.scores.get(self.parent.level,0)
        curPlayer.scores[self.parent.level]=shotNum+1

        self.abandonTextDL=textDraw.makeCenteredStringDL(400,25,4,"click at any time to abandon shot")
        


    def update(self, dt):
        self.parent.update(dt)

        dt=dt/1000.0

        parent=self.parent

        (bLandedOnTarget,
         bLandedOnPlanet,
         bStillInFlight,
         p,
         newPoint,
         newVel)=miniSim.evaluateTick(parent.packageX,
                                      parent.packageY,
                                      parent.packageVX,
                                      parent.packageVY,
                                      parent.physicsObjects,
                                      self.flightTime,
                                      dt,
                                      playerMgr.getCurrentPlanet())

        self.flightTime += dt

        parent.packageVX = newVel[0]
        parent.packageVY = newVel[1]
        parent.packageX  = newPoint[0]
        parent.packageY  = newPoint[1]

        if p:
            resetZoom()
            if p.get("target",False):
                soundMgr.playSound("cheer")
                playerMgr.getCurrentPlayer().finishedLevel=True

                if playerMgr.nextPlayer():
                    fsmMgr.popState()
                    fsmMgr.pushState(stateAim.StateAim(None))            
                else:
                    fsmMgr.popState()
                    if options.gUnattended:
                        self.parent.pickRandomLevel()
                        fsmMgr.pushState(stateAim.StateAim(None))
                    else:
                        fsmMgr.pushState(stateLevelScore.StateLevelScore(None))
            elif (p.get("gaseous",False) or
                  p.get("dust",False)):
                # oh no!
                bNewPlayer=playerMgr.nextPlayer()
                assert(bNewPlayer)
                fsmMgr.popState()
                fsmMgr.pushState(stateAim.StateAim(None))            

            else:
                playerMgr.getCurrentPlayer().curPlanet=p
                bNewPlayer=playerMgr.nextPlayer()
                assert(bNewPlayer)
                
                fsmMgr.popState()
                fsmMgr.pushState(stateAim.StateAim(None))

            glDeleteLists(self.abandonTextDL, 1)
            return

        if (parent.packageX>1600 or
            parent.packageX<-800 or
            parent.packageY>1500 or
            parent.packageY<-900):
            resetZoom()
            bNewPlayer=playerMgr.nextPlayer()
            assert(bNewPlayer)
            fsmMgr.popState()
            fsmMgr.pushState(stateAim.StateAim(None))

        self.logPosition()
        #self.soundPing()

    def logPosition(self):
        doLog=False
        if len(self.locs)==0:
            doLog=True
        else:
            lastTime,pos=self.locs[-1]
            if self.flightTime-lastTime>=LOC_INCREMENT:
                doLog=True

        if doLog:
            self.locs.append((self.flightTime,
                              (self.parent.packageX,
                               self.parent.packageY)))

    def soundPing(self):
        if not options.gSound:
            return

        vx=self.parent.packageVX
        vy=self.parent.packageVY

        vMag=math.sqrt(vx*vx+vy*vy)
        pingPeriod=statePutt.BASE_SPEED/vMag*BASE_PING_PERIOD

        if self.flightTime-self.lastPingTime>pingPeriod:
            soundMgr.playSound("ping")
            self.lastPingTime=self.flightTime


            

    def handleMouseButton(self, bDown, buttonIndex):
        #trigger this at the beginning of the click

        if self.flightTime>1.0 and bDown==True:
            self.exitMouseDown=True
            return True
        elif ((not bDown) and
              self.exitMouseDown):
            resetZoom()
            bNewPlayer=playerMgr.nextPlayer()
            assert(bNewPlayer)
            fsmMgr.popState()
            fsmMgr.pushState(stateAim.StateAim(None))
            return True
        return False
    
    def draw(self):
        px=self.parent.packageX
        py=self.parent.packageY

        zoomOut(px,py)
        
        # first, let our parent draw the map
        self.parent.draw()

        if DRAW_ACCELERATION:
            glBegin(GL_LINES)
            glVertex(px,py)
            glVertex(px+self.ax*2,py+self.ay*2)
            glEnd()

        duration=4.0
        
        glColor3f(1.0, 0.5, 0.0)
        baseR=0.8
        baseG=0.7
        baseB=0.0
        
        for loc in self.locs:
            tm,pos=loc
            if self.flightTime-tm>duration:
                continue
            fade=(self.flightTime-tm)/duration
            bright=1.0-fade
            glColor3f(baseR*bright,
                      baseG*bright,
                      baseB*bright)

            px,py=pos
            glBegin(GL_LINE_LOOP)
            glVertex(px-3,py)
            glVertex(px,py-3)
            glVertex(px+3,py)
            glVertex(px,py+3)
            glEnd()

        px=self.parent.packageX
        py=self.parent.packageY

        glColor3f(1,1,1)
        glBegin(GL_LINE_LOOP)
        glVertex(px-5,py)
        glVertex(px,py-5)
        glVertex(px+5,py)
        glVertex(px,py+5)
        glEnd()            

        if self.flightTime>10.0:
            resetZoom()
            glColor3f(0.5, 1.0, 0.5)            
            glCallList(self.abandonTextDL)



    

def zoomOut(px, py):
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()

    minX=min(0,px-50)
    maxX=max(800, px+50)
    minY=min(0,py-50)
    maxY=max(600, py+50)

    centerX=(maxX+minX)/2
    centerY=(maxY+minY)/2

    wX=maxX-minX
    wY=maxY-minY

    aspect=4.0/3.0
    if wX>wY*aspect:
        #need to boost wY
        wY=wX/aspect
        minY=centerY-wY/2
        maxY=centerY+wY/2
    elif wY>wX/aspect:
        #need to boost wX
        wX=wY*aspect
        minX=centerX-wX/2
        maxX=centerX+wX/2
    
    gluOrtho2D(minX, maxX,
               minY, maxY)
               
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
        
def resetZoom():
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluOrtho2D(0,800,0,600)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    
