import math
import sys

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import mouseInfo

import statePlayAd
import stateDialog
import level1
import stateAim
import edgesMenu
import widgets
import tutorialProgress
import playerMgr
import stateMainMenu
import screenInfo
import fade       
import stateMainMenu
import stateEditPlayerDesc

import tutorialProgress
import textDraw
import soundMgr

import levelIntros

NUM_DISPLAYED = 10
gPlayerList=[]
g_scrollOffset=0

g_thumbWidget=None

class PlayerDesc:
    def __init__(self, name, isHuman):
        self.name=name
        self.isHuman=isHuman
        self.removeButton=None
        self.editButton=None
        self.position=-1

    def removeButtonCB(self):
        pressRemoveIndex(self.position)

    def editButtonCB(self):
        pressEditIndex(self.position)

def isNameInUse(name):
    for p in gPlayerList:
        if p.name==name:
            return True
    return False

def pressNewHuman():
    num=1
    name="Human %d"%num
    while isNameInUse(name):
        num += 1
        name="Human %d"%num

    gPlayerList.append(PlayerDesc(name, 1))

    positionScrollBottom()

def pressNewAI():
    num=1
    name="AI %d"%num
    while isNameInUse(name):
        num += 1
        name="AI %d"%num

    gPlayerList.append(PlayerDesc(name, 0))

    positionScrollBottom()

def pressRemove():
    gPlayerList.pop()
    positionScrollAfterRemoval()

def pressRemoveIndex(i):
    gPlayerList.remove(gPlayerList[i])
    positionScrollAfterRemoval()

def pressEditIndex(i):
    fsmMgr.pushState(stateEditPlayerDesc.StateEditPlayerDesc(gPlayerList[i]))

def positionScrollBottom():
    global g_scrollOffset

    pll=len(gPlayerList)

    if (pll>NUM_DISPLAYED):
        g_scrollOffset=pll-NUM_DISPLAYED

    updateThumb()
    updateButtons()

def positionScrollAfterRemoval():
    global g_scrollOffset

    pll=len(gPlayerList)

    if (g_scrollOffset+NUM_DISPLAYED>pll):
        g_scrollOffset=pll-NUM_DISPLAYED

    if g_scrollOffset<0:
        g_scrollOffset=0

    updateThumb()
    updateButtons()

def dynamicButtons():
    buttonList=[]
    
    for p in gPlayerList:
        if p.removeButton:
            buttonList.append(p.removeButton)
        if p.editButton:
            buttonList.append(p.editButton)

    return buttonList

    

def updateButtons():
    for i in range(len(gPlayerList)):
        gPlayerList[i].position=i
        if i<g_scrollOffset or i>= g_scrollOffset+NUM_DISPLAYED:
            gPlayerList[i].removeButton=None
            gPlayerList[i].editButton=None
        else:
            listIndex=i-g_scrollOffset
            y=500-30*listIndex
            sz=5
            gPlayerList[i].removeButton=widgets.Button(600,y,
                                                       sz,"remove",
                                                       gPlayerList[i].removeButtonCB)
            gPlayerList[i].editButton=widgets.Button(525,y,
                                                     sz, "edit",
                                                     gPlayerList[i].editButtonCB)


def updateThumb():
    if len(gPlayerList)<=NUM_DISPLAYED:
        f=0.0
    else:
        f=g_scrollOffset/float(len(gPlayerList)-NUM_DISPLAYED)
    g_thumbWidget.setFrac(f)
    
def countHumans():
    h=0
    for p in gPlayerList:
        if p.isHuman:
            h += 1
    return h

def countAI():
    h=0
    for p in gPlayerList:
        if not p.isHuman:
            h += 1
    return h

    
def pressPlay():
    fsmMgr.popAll()

    tutorialProgress.gTutorial=False
    #playerMgr.initPlayers(countHumans(), countAI())
    playerMgr.initPlayers(0,0)
    for p in gPlayerList:
        playerMgr.addPlayer(p.name,p.isHuman)
    
    level1.thisLevel.setLevel(10)
    
    fsmMgr.pushState(level1.thisLevel)
    fsmMgr.pushState(stateAim.StateAim(None))

    introText=levelIntros.intros.get(10,None)
    if introText:
        title="Level %2d"%1
        fsmMgr.pushState(stateDialog.StateDialog((title,introText)))
                
def pressScrollUp():
    global g_scrollOffset

    if g_scrollOffset>0:
        g_scrollOffset-=1

    updateThumb()
    updateButtons()

def pressScrollDown():
    global g_scrollOffset

    g_scrollOffset+=1

    positionScrollAfterRemoval()
    
    

def pressCancel():
    fsmMgr.popAll()
    fsmMgr.pushState(stateMainMenu.StateMainMenu(None))


class StateNewGameMenuDeluxe(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)

        self.clicked=False

        self.left=40
        self.right=760
        self.top=560
        self.bottom=40

        global g_scrollOffset
        g_scrollOffset=0
        global gPlayerList
        gPlayerList=[]

        global g_thumbWidget
        g_thumbWidget=widgets.Thumb(730,45,32,488,261)
        
        self.widgets=[widgets.Button(250,120,8,"Add Human",pressNewHuman),
                      widgets.Button(550,120,8,"Add AI",pressNewAI),
                      widgets.Button(250,60,8,"CANCEL", pressCancel),
                      widgets.Button(550,60,8,"OK", pressPlay),
                      widgets.Button(730,488,8,"\x01", pressScrollUp),
                      widgets.Button(730,229,8,"\x02", pressScrollDown),
                      g_thumbWidget
                      ]

        myWidths=[250,250,
                  250,250,
                  45,45,45]

        assert(len(self.widgets)==len(myWidths))
        
        for i in range(len(self.widgets)):
            self.widgets[i].width=myWidths[i]

        for w in self.widgets:
            w.x=w.x-w.width/2
            w.makeDL()

        pressNewHuman()
        pressNewAI()
        pressNewAI()

    def __del__(self):
        global g_thumbWidget
        g_thumbWidget=None


    def update(self, dt):
        dt=dt/1000.0

        global g_scrollOffset
        mouseInfo.readMouse()
        for w in self.widgets + dynamicButtons():
            w.update(dt)

        if len(gPlayerList)>NUM_DISPLAYED:
            tf=g_thumbWidget.frac
            thumbOffset=int(tf*(len(gPlayerList)-NUM_DISPLAYED))
            if thumbOffset != g_scrollOffset:
                g_scrollOffset=thumbOffset
                updateButtons()
            
    def draw(self):
        glColor3f(0,0,0)
        glBegin(GL_QUADS)
        glVertex(self.left,self.bottom)
        glVertex(self.right,self.bottom)
        glVertex(self.right,self.top)
        glVertex(self.left,self.top)
        glEnd()

        glColor3f(*fade.DIM_COLOR)
        glBegin(GL_LINE_LOOP)
        glVertex(self.left,self.bottom)
        glVertex(self.right,self.bottom)
        glVertex(self.right,self.top)
        glVertex(self.left,self.top)
        glEnd()
        
        for w in self.widgets:
            w.draw()

        textDraw.drawCenteredString(400, 525,
                                    fade.DIM_COLOR,
                                    8,
                                    "NEW GAME")

        first=g_scrollOffset
        last=min(g_scrollOffset+NUM_DISPLAYED,
                 len(gPlayerList))
        
        for playerIndex in range(first,last):
            listIndex=playerIndex-first
            playerDesc=gPlayerList[playerIndex]
                
            if playerDesc.isHuman:
                color=(.9, .2, .2)
            else:
                color=(.2, .2, .9)
                
            textDraw.drawString(75,500-30*listIndex,
                                color,
                                10,
                                playerDesc.name)

            if playerDesc.removeButton:
                playerDesc.removeButton.draw()

            if playerDesc.editButton:
                playerDesc.editButton.draw()
            
        if fsmMgr.isTop(self):
            widgets.drawCursor()


            

    def handleMouseButton(self, bDown, buttonIndex):
        mouseInfo.readMouse()
        if bDown and buttonIndex==1:
            for w in self.widgets + dynamicButtons():
                if w.isMouseInMe():
                    soundMgr.playSound("ping")
                    w.callbackFunc()
        return True

    def done(self):
        fsmMgr.popState()

    

