from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import textDraw

import mouseInfo
import fade

import soundMgr


class Button:
    def __init__(self, x, y, textSize, label, func):
        self.x=x
        self.y=y
        self.textSize=textSize
        self.label=label
        self.callbackFunc=func

        self.width=textDraw.stringWidth(textSize,label)+2*textSize
        self.fadeTime=0
        self.color=fade.DIM_COLOR

        self.state=False
        self.dispList=None
        self.makeDL()

    def __del__(self):
        if not (self.dispList is None):
            if glDeleteLists:
                glDeleteLists(self.dispList, 1)
            self.dispList=None

    def setWidth(self, w):
        self.width=w
        self.makeDL()

    def makeDL(self):
        if not (self.dispList is None):
            glDeleteLists(self.dispList,1)
        self.dispList=glGenLists(1)
        glNewList(self.dispList, GL_COMPILE)

        glBegin(GL_LINE_LOOP)
        glVertex(self.x,self.y)
        glVertex(self.x+self.width,self.y)
        glVertex(self.x+self.width,self.y+self.textSize*4)
        glVertex(self.x,self.y+self.textSize*4)
        glEnd()

        textDraw.drawCenteredString(self.x+self.width/2,
                                    self.y,
                                    self.color,
                                    self.textSize,
                                    self.label)

        glEndList()
        

    def draw(self):
        glColor(*self.color)

        glCallList(self.dispList)
        
        
    def update(self, dt):
        if self.state:
            self.color=fade.BRIGHT_COLOR
        
        elif self.isMouseInMe():
            self.fadeTime+=dt
            self.color=fade.computeColor(0.0, self.fadeTime)
            
            
        else:
            self.fadeTime=0
            self.color=fade.DIM_COLOR

    def isMouseInMe(self):
        mx=mouseInfo.gMouseX
        my=mouseInfo.gMouseY

        return (mx>self.x and mx<self.x+self.width and
                my>self.y and my<self.y+4*self.textSize)
            


class Thumb:
    def __init__(self, x, width, height, topY, bottomY):
        self.x=x
        self.topY=topY-height
        self.bottomY=bottomY
        self.width=width
        self.height=height
        self.callbackFunc=self.mouseDown

        self.range=self.bottomY-self.topY
        self.setFrac(0)

        self.color=fade.DIM_COLOR

        self.isMouseDown=False
        
    def __del__(self):
        pass

    def setWidth(self, w):
        self.width=w

    def makeDL(self):
        pass

    def draw(self):
        glColor(*self.color)

        glBegin(GL_LINE_LOOP)
        glVertex(self.x,self.y)
        glVertex(self.x+self.width,self.y)
        glVertex(self.x+self.width,self.y+self.height)
        glVertex(self.x,self.y+self.height)
        glEnd()

    def mouseDown(self):
        self.isMouseDown=True

    def setFrac(self, frac):
        self.frac=frac
        self.y=self.range*self.frac+self.topY

        
    def update(self, dt):
        b1=pygame.mouse.get_pressed()[0]

        if not b1:
            self.isMouseDown=False
            
        if self.isMouseDown:
            self.color=fade.BRIGHT_COLOR

            newY=mouseInfo.gMouseY
            newFrac=(newY-self.height/2-self.topY)/float(self.range)
            newFrac=max(0.0, newFrac)
            newFrac=min(1.0, newFrac)
            self.setFrac(newFrac)
            
        elif self.isMouseInMe():
            self.fadeTime+=dt
            self.color=fade.computeColor(0.0, self.fadeTime)
            
        else:
            self.fadeTime=0
            self.color=fade.DIM_COLOR

    def isMouseInMe(self):
        mx=mouseInfo.gMouseX
        my=mouseInfo.gMouseY

        return (mx>self.x and mx<self.x+self.width and
                my>self.y and my<self.y+self.height)
            
            
            
                 
def drawCursor():
    cursor_radius=20
    glColor3f(0, 0.7, 1)
    glBegin(GL_LINES)
    glVertex2f(mouseInfo.gMouseX-cursor_radius,
               mouseInfo.gMouseY)
    glVertex2f(mouseInfo.gMouseX+cursor_radius,
               mouseInfo.gMouseY)
    glVertex2f(mouseInfo.gMouseX,
               mouseInfo.gMouseY-cursor_radius)
    glVertex2f(mouseInfo.gMouseX,
               mouseInfo.gMouseY+cursor_radius)
    glEnd()

    
    
    
